# Copyright (C) 2022, the ixpeobssim team.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from enum import Enum, unique

from matplotlib.lines import Line2D

from ixpeobssim.utils.matplotlib_ import plt


@unique
class TWG(Enum):

    """Definition of the TWGs.
    """

    PWN = 'PWN and radio pulsars'
    SNR = 'SNR'
    ASMBH = 'Accreting stellar-mass BH'
    AWDNS = 'Accreting WD and NS'
    MAGNETAR = 'Magnetars'
    RQAGN = 'Radio-quiet AGN and Sgr A*'
    BLAZAR = 'Blazars and radiogalaxies'


# Collapse the enum into a list, which will be handy later.
TWG_LIST = list(TWG)

# A few matplotlib-related things, for plotting purposes.
_COLORS = plt.rcParams['axes.prop_cycle'].by_key()['color']
COLOR_DICT = {cls: _COLORS[i] for i, cls in enumerate(TWG_LIST)}
COLOR_DICT[None] = 'black'
SOURCE_LEGEND_HANDLES = []
for i, cls in enumerate(TWG):
    handle = Line2D([0], [0], marker='o', color='w', markerfacecolor=COLOR_DICT[cls],
        markersize=10., label=cls.value)
    SOURCE_LEGEND_HANDLES.append(handle)



# This is compiled by hand and maintained to match the evolving ARTL.
#
# The dictionary is indexed by source name and the fields are (ra, dec, twg).
_TARGET_DATA = {
    '1ES 0229+200': (38.20256456, 20.2881926, TWG.BLAZAR),
    '1ES 1959+650': (299.99938562, 65.14851421, TWG.BLAZAR),
    '1RXS J170849.0': (257.20416667, -40.15277778, TWG.MAGNETAR),
    '3C 273': (187.27791535, 2.05238857, TWG.BLAZAR),
    '3C 279': (194.04652688, -5.7893144, TWG.BLAZAR),
    '3C 454.3': (343.49061658, 16.14821142, TWG.BLAZAR),
    '4U 0142+61': (26.59253, 61.75106, TWG.MAGNETAR),
    '4U 1626-67': (248.06996, -67.46091, TWG.AWDNS),
    '4U 1630-472': (248.506708, -47.393, TWG.ASMBH),
    '4U 1820-303': (275.919042, -30.361278, TWG.AWDNS),
    'BL Lac': (330.68038074, 42.27777178, TWG.BLAZAR),
    'Cas A': (350.85, 58.815, TWG.SNR),
    'Cen A': (201.36506288, -43.01911267, TWG.BLAZAR),
    'Cen X-3': (170.31288349, -60.62378542, TWG.AWDNS),
    'Circinus galaxy': (213.291458, -65.339222, TWG.RQAGN),
    'Crab': (83.63308333, 22.0145, TWG.PWN),
    'Cyg X-1': (299.59031591, 35.20160625, TWG.ASMBH),
    'Cyg X-2': (326.17147688, 38.32140718, TWG.AWDNS),
    'Cyg X-3': (308.107417, 40.95775, TWG.AWDNS),
    'GRO J1008-57': (152.4456843, -58.29322329, TWG.AWDNS),
    'GRS 1915+105': (288.798149, 10.945807, TWG.ASMBH),
    'GS 1826-238': (277.3675, -23.79694444, TWG.AWDNS),
    'GX 301-2': (186.65650295, -62.77034989, TWG.AWDNS),
    'GX 9+9': (262.93404167, -16.96136111, TWG.AWDNS),
    'Her X-1': (254.45754617, 35.34235762, TWG.AWDNS),
    'IC 4329A': (207.33025303, -30.3095067, TWG.RQAGN),
    'J0211+1051': (32.804905, 10.859666, TWG.BLAZAR),
    'LMC X-1': (84.91178513, -69.74320321, TWG.ASMBH),
    'MCG-5-23-16': (146.917319, -30.948734, TWG.RQAGN),
    'MSH 15-52' : (228.320833, -59.0816667, TWG.PWN),
    'Mrk 421': (166.113808, 38.20883287, TWG.BLAZAR),
    'Mrk 501': (253.46756952, 39.76016915, TWG.BLAZAR),
    'NGC 4151': (182.63573326, 39.40585098, TWG.RQAGN),
    'S5 0716+714': (110.47270204, 71.34343391, TWG.BLAZAR),
    'Sgr A complex': (266.41681662, -29.00782497, TWG.RQAGN),
    'SN 1006': (225.59208, -42.09694, TWG.SNR),
    'Tycho': (6.308299, 64.144305, TWG.SNR),
    'Vela Pulsar': (128.5, -45.83333333, TWG.PWN),
    'Vela X-1': (135.52858781, -40.55469345, TWG.AWDNS),
    'X Persei': (58.84615783, 31.04584604, TWG.AWDNS),
    'XTE J1701-462': (255.243583, -46.185722, TWG.AWDNS),
    'GRB 221009A': (288.254, 19.809, None),
    'EXO 2030+375': (308.06364577422, 37.63745639578999, TWG.AWDNS),
    'NGC 4151': (182.63573325577997, 39.40585097987, TWG.RQAGN),
    'PSR B0540-69': (85.04666687101583, -69.33172182217471, TWG.PWN),
    'IC 4329A': (207.33025075050998, -30.30950383764, TWG.RQAGN),
    '1ES 0229+200': (38.20256203112, 20.28819075483, TWG.BLAZAR),
    'PG 1553+113': (238.92935002022, 11.19010156943, TWG.BLAZAR),
    'LS V +44 17': (70.24720676553, 44.53034945816, TWG.AWDNS),
    'GX 5-1': (270.29054502533995, -25.07892228369001, TWG.AWDNS),
    'SGR 1806-20': (272.16385, -20.41114, TWG.MAGNETAR),
    'SS 433': (287.95651987395, 4.98272975768, TWG.ASMBH),
    '4U 1957+115': (299.85003093835, 11.70829438147, TWG.ASMBH),
    'S4 0954+65': (149.69685494674, 65.5652275985, TWG.BLAZAR),
    '1E 2259+586': (345.28391, 58.87904, TWG.MAGNETAR),
    'LMC X-3': (84.73596816896, -64.08425498264, TWG.ASMBH),
    'Swift J0243.6+6124': (40.91843869563, 61.43437707134001, TWG.AWDNS),
    'RCW 86': (220.1208, -62.645, TWG.SNR),
    'Cir X-1': (230.17022189460832, -57.16672406639805, TWG.AWDNS),
    '4U 1624-49': (247.0117708, -49.1985028, TWG.AWDNS),
    'RX J1713.7-3946': (258.1125, -39.6867, TWG.SNR),
    'Sco X-1': (244.97945527876, -15.64028268515, TWG.AWDNS),
    'Swift J1727.8-1613': (261.9417, -16.21472, TWG.ASMBH),
    'G21.5-0.9': (278.39005, -10.5688028, TWG.PWN),
    'GX 13+1': (273.631458, -17.157417, TWG.AWDNS),
    'PKS 2155-304': (329.71693843745, -30.22558845772, TWG.BLAZAR),
    'Vela Jr': (133.0, -46.333, TWG.SNR),
    'SMC X-1': (19.27144053683, -73.44333744671, TWG.AWDNS),
    'NGC 1068': (40.66962152891999, -0.01329435839, TWG.RQAGN)
}



# These were copied and formatted by hand from an excel spreadsheet maintained by
# Steve Odell. (At the cost of a small risk of editing errors, we preferred not
# read the excel file directly, as this would have added another dependence on
# ixoeobssim).
#
# The header row of the excel file reads:
#
# Source Name | (TWG() | Expos [d] | UID | Start | Stop | (Internal) | (Public) | Comments
#
# Note that we explicitly left out the fields for the TWG, as well as the
# internal and external data release.
_Y1_ARTL_DATA = (
    ('Cas A', 11.57, 1001301, '2022-01-11T11:45', '2022-01-29T12:34', ''),
    ('Cen X-3',	1.16, 1006501, '2022-01-29T13:10', '2022-01-31T07:10', ''),
    ('4U 0142+61',	6.94, 1003201, '2022-01-31T07:36', '2022-02-14T23:48', '1 of 2'),
    ('Cen A', 1.16, 1004301, '2022-02-15T00:38', '2022-02-17T13:33', ''),
    ('Her X-1', 2.31, 1001801, '2022-02-17T14:11', '2022-02-21T15:50', '1 of 2'),
    ('Crab', 0.58, 1001001, '2022-02-21T16:34', '2022-02-22T18:23', '1 of 2'),
    ('Her X-1', 1.16, 1001802, '2022-02-22T19:07', '2022-02-24T19:24', '2 of 2'),
    ('4U 0142+61',	1.97, 1003202,'2022-02-24T19:47', '2022-02-27T18:51', '2 of 2'),
    ('Sgr A complex', 4.05, 1003401, '2022-02-27T19:37', '2022-03-06T23:47', '1 of 2'),
    ('Crab', 0.58, 1001002, '2022-03-07T00:38', '2022-03-08T02:17', '2 of 2'),
    ('Mrk 501', 1.16, 1004501, '2022-03-08T02:59', '2022-03-10T08:30', ''),
    ('Sgr A complex', 7.52, 1003402, '2022-03-10T08:30', '2022-03-23T01:42', '2 of 2'),
    ('4U 1626-67', 2.31, 1002701, '2022-03-24T01:59', '2022-03-27T05:19', ''),
    ('Mrk 501', 1.16, 1004601, '2022-03-27T05:59', '2022-03-29T07:03', ''),
    ('GS 1826-238',	1.16, 1002801, '2022-03-29T07:25', '2022-03-31T08:57', ''),
    ('S5 0716+714',	4.63, 1005301, '2022-03-31T09:42', '2022-04-05T19:29', ''),
    ('Vela Pulsar', 5.79, 1001201, '2022-04-05T20:11', '2022-04-15T18:02', '1 of 2'),
    ('Vela X-1', 3.47, 1002501, '2022-04-15T18:11', '2022-04-21T12:17', ''),
    ('Vela Pulsar', 5.24, 1001202, '2022-04-21T12:24', '2022-04-30T10:02', '2 of 2'),
    ('Cyg X-2',	1.16, 1001601, '2022-04-30T11:04', '2022-05-02T11:07', ''),
    ('Cyg X-2', 0.57, 1006601, '2022-05-02T11:11', '2022-05-03T11:13', 'off-set'),
    ('1ES 1959+650', 0.58, 1006201, '2022-05-03T11:29', '2022-05-04T09:49', ''),
    ('Mrk 421',	1.16, 1003701, '2022-05-04T10:11', '2022-05-06T10:50', ''),
    ('BL Lac', 4.63, 1006301, '2022-05-06T11:30', '2022-05-14T12:27', ''),
    ('MCG-5-23-16', 0.58, 1003301, '2022-05-14T13:17', '2022-05-15T14:55', '1 of 2'),
    ('Cyg X-1',	3.47, 1002901, '2022-05-15T15:45', '2022-05-21T17:52', ''),
    ('MCG-5-23-16', 5.21, 1003302, '2022-05-21T18:42', '2022-05-31T03:47', '2 of 2'),
    ('3C 454.3', 1.16, 1005401, '2022-05-31T04:36', '2022-06-02T08:40', ''),
    ('3C 273', 1.16, 1005901, '2022-06-02T08:53', '2022-06-04T10:48', ''),
    ('Mrk 421',	1.16, 1003801, '2022-06-04T11:05', '2022-06-06T10:57', ''),
    ('1ES 1959+650', 0.58, 1006101, '2022-06-06T11:20', '2022-06-07T08:38', 'adjust'),
    ('Mrk 421',	1.16, 1003901, '2022-06-07T09:01', '2022-06-09T09:39', ''),
    ('1ES 1959+650', 2.46, 1006001, '2022-06-09T10:02', '2022-06-12T20:25', ''),
    ('3C 279', 3.24, 1005701, '2022-06-12T21:05', '2022-06-18T20:21', ''),
    ('Cyg X-1', 1.16, 1250101, '2022-06-18T21:03', '2022-06-21T21:03', 'ToO'),
    ('Tycho', 11.57, 1001401, '2022-06-21T21:22', '2022-07-07T00:00', ''),
    ('Cen X-3', 2.31, 1250201, '2022-07-04T06:22', '2022-07-07T12:44', 'ToO'),
    ('BL Lac', 1.44, 1006701, '2022-07-07T13:35', '2022-07-09T23:12', ''),
    ('Mrk 501', 1.16, 1004701, '2022-07-09T23:33', '2022-07-12T00:40', ''),
    ('Circinus galaxy', 9.26, 1003501, '2022-07-12T01:23', '2022-07-25T02:05', ''),
    ('SN 1006', 2.31, 1001501, '2022-07-25T02:20', '2022-07-29T02:19', 'Seg 1/2 Wrong coordinates'),
    ('GX 301-2', 3.47, 1002601, '2022-07-29T02:35', '2022-08-03T06:01', ''),
    ('SN 1006', 1.27, 1001502, '2022-08-03T06:16', '2022-08-05T14:18', 'Seg 2/2: Obs UID=1001599 Wrong coordinates'),
    ('SN 1006', 7.99, 1006801, '2022-08-05T14:21', '2022-08-19T12:32', ''),
    ('X Persei', 1.61, 1001701, '2022-08-19T13:22', '2022-08-22T05:39', 'Seg 1/2 interrupted by ToO'),
    ('4U 1630-472', 0.97, 1250301, '2022-08-22T06:28', '2022-08-23T22:37', 'Wrong coordinates'),
    ('4U 1630-472', 5.83, 1250401, '2022-08-23T22:47', '2022-09-02T18:54', 'ToO'),
    ('MSH 15-52', 10.48, 1001101, '2022-09-02T19:06', '2022-09-16T20:19', ''),
    ('X Persei', 1.28, 1001702, '2022-09-16T21:07', '2022-09-19T03:41', 'Seg 2/2 Obs UID=1001799 post ToO'),
    ('1RXS J170849.0', 0.08, 1003101, '2022-09-19T04:44', '2022-09-19T07:20', 'Seg 1/3'),
    ('1RXS J170849.0', 5.76, 1003102, '2022-09-19T17:30', '2022-09-29T12:01', 'Seg 2/3'),
    ('XTE J1701-462', 0.58, 1250601, '2022-09-29T12:09',	'2022-09-30T12:08', ''),
    ('1RXS J170849.0', 4.58, 1003103, '2022-09-30T12:14', '2022-10-08T11:19', 'Seg 3/3: Obs UID=1003199'),
    ('XTE J1701-462', 0.58, 1250602, '2022-10-08T11:27', '2022-10-09T11:25', ''),
    ('GX 9+9', 1.16, 1002401, '2022-10-09T11:41', '2022-10-11T13:13', ''),
    ('4U 1820-303', 0.12, 2002301, '2022-10-11T13:24', '2022-10-11T22:55', ''),
    ('GRB 221009A', 1.16, 2250101, '2022-10-11T23:15', '2022-10-14T00:46', 'ToO fast'),
    ('Cyg X-3',	3.23, 2001801, '2022-10-14T01:01', '2022-10-19T14:14',	'Seg 1/2'),
    ('LMC X-1',	6.94, 2001901, '2022-10-19T15:01', '2022-10-28T04:39', ''),
    ('1ES 1959+650', 2.31, 2004801, '2022-10-28T05:29', '2022-10-31T12:21', ''),
    ('Cyg X-3', 3.34, 2001802, '2022-10-31T12:35', '2022-11-06T09:34', 'Seg 2/2: Obs UID=2002899'),
    ('MCG-5-23-16', 3.71, 2003201, '2022-11-06T10:24', '2022-11-13T03:04', 'Seg 1/3'),
    ('GRO J1008-57', 0.98, 2003501, '2022-11-13T03:19', '2022-11-14T17:45', 'Re-released (gain issue)'),
    ('MCG-5-23-16', 1.91, 2003202, '2022-11-14T18:00', '2022-11-18T03:33', 'Seg 2/3'),
    ('GRO J1008-57', 1.23, 2003601, '2022-11-18T03:47', '2022-11-20T02:15', 'Late release (gain issue)'),
    ('MCG-5-23-16',	1.98, 2003203, '2022-11-20T02:30', '2022-11-23T18:29', 'Seg 3/3: Obs UID=2003299'),
    ('EXO 2030+375', 2.10, 2250201, '2022-11-23T19:19', '2022-11-27T13:10',	'ToO'),
    ('BL Lac', 1.89, 2005901, '2022-11-27T13:21', '2022-11-30T21:35', ''),
    ('Vela X-1', 3.19, 2005801, '2022-11-30T22:36', '2022-12-06T14:51', ''),
    ('Mrk 421',	0.87, 2004401, '2022-12-06T15:16', '2022-12-08T03:56', ''),
    ('NGC 4151', 7.63, 2003101, '2022-12-08T04:06', '2022-12-21T14:51', ''),
    ('Tycho', 2.57, 2001601, '2022-12-21T15:16', '2022-12-25T09:48', ''),
    ('Cyg X-3', 2.43, 2250301, '2022-12-25T10:05', '2022-12-29T17:45', 'ToO (slow)'),
    )

_Y2_ARTL_DATA = (
    ('PSR B0540-69', 5.48, 2001201, '2022-12-29T18:33', '2023-01-05T12:46', 'Seg 1/2'),
    ('IC 4329A', 5.44, 1003601, '2023-01-05T13:09', '2023-01-15T09:12', ''),
    ('1ES 0229+200', 1.78, 1006401, '2023-01-15T10:02', '2023-01-18T16:44', 'Seg 1/2'),
    ('Her X-1', 1.74, 2003801, '2023-01-18T17:26', '2023-01-21T21:11', ''),
    ('PSR B0540-69', 3.89, 2001202, '2023-01-21T21:59', '2023-01-27T00:15', 'Seg 2/2: Obs UID=2001299'),
    ('1ES 0229+200', 3.01, 1006402, '2023-01-27T00:53', '2023-02-01T14:44', 'Seg 2/2: Obs UID=1006499'),
    ('PG 1553+113', 1.16, 2004901, '2023-02-01T15:31', '2023-02-03T19:01', 'Seg 1/2'),
    ('Her X-1', 2.89, 2004001, '2023-02-03T19:16', '2023-02-08T22:45', ''),
    ('PG 1553+113', 0.32, 2004902, '2023-02-08T23:00', '2023-02-09T14:43', 'Seg 2/2: Obs UID=2004999 (stop for ToO)'),
    ('LS V +44 17', 1.54, 2250401, '2023-02-09T15:27', '2023-02-12T09:27', 'ToO (slow)'),
    ('Mrk 501', 1.16, 2004601, '2023-02-12T10:06', '2023-02-14T12:02', ''),
    ('MSH 15-52', 4.76, 2001301, '2023-02-14T12:41', '2023-02-21T23:50', 'Seg 1/2'),
    ('Crab', 0.87, 2001001, '2023-02-22T00:37', '2023-02-23T15:13', 'Seg 1/2'),
    ('LS V +44 17', 4.63, 2250501, '2023-02-23T15:26', '2023-03-03T15:51', 'ToO (slow)'),
    ('SN 1006', 4.05, 2001701, '2023-03-03T16:42', '2023-03-10T18:42', ''),
    ('4U 1630-472', 1.74, 2250601, '2023-03-10T18:53', '2023-03-13T19:03', 'ToO'),
    ('MSH 15-52', 3.35, 2001302, '2023-03-13T19:14', '2023-03-19T00:14', 'Seg 2/2: Obs UID=2001399'),
    ('Mrk 501', 1.16, 2004501, '2023-03-19T00:54', '2023-03-21T03:37', ''),
    ('GX 5-1', 0.58, 2002701, '2023-03-21T03:59', '2023-03-22T05:21', 'Seg 1/2'),
    ('SGR 1806-20', 5.71, 2002801, '2023-03-22T05:27', '2023-04-01T18:57', 'Seg 1/2'),
    ('Crab', 0.87, 2001002, '2023-04-01T19:49', '2023-04-03T10:15', 'Seg 2/2: Obs UID=2001099'),
    ('SGR 1806-20', 5.71, 2002802, '2023-04-03T11:07', '2023-04-13T23:01', 'Seg 2/2: Obs UID=2002899'),
    ('GX 5-1', 0.58, 2002702, '2023-04-13T23:09', '2023-04-15T00:43', 'Seg 2/2: Obs UID=2002799'),
    ('4U 1820-303', 1.08, 2002302, '2023-04-15T00:51', '2023-04-16T23:17', ''),
    ('Mrk 501', 1.08, 2004701, '2023-04-16T23:42', '2023-04-18T23:21', ''),
    ('SS 433', 7.52, 2006701, '2023-04-18T23:40', '2023-05-02T22:51', 'Seg 1/2'),
    ('Cyg X-1', 0.28, 2008201, '2023-05-02T23:07', '2023-05-03T11:45', ''),
    ('SS 433', 3.42, 2006702, '2023-05-03T12:01', '2023-05-09T10:30', 'Seg 2/2: Obs UID=2006799'),
    ('Cyg X-1', 0.4, 2008301, '2023-05-09T10:45', '2023-05-10T04:13', ''),
    ('PSR B0540-69', 1.39, 2008801, '2023-05-10T04:59', '2023-05-12T01:57', ''),
    ('4U 1957+115', 6.82, 2006601, '2023-05-12T02:40', '2023-05-24T19:35', ''),
    ('Cyg X-1', 0.35, 2008401, '2023-05-24T19:48', '2023-05-25T11:39', ''),
    ('S4 0954+65', 5.85, 2009001, '2023-05-25T12:02', '2023-06-02T22:56', ''),
    ('1E 2259+586', 7.0, 2007801, '2023-06-02T23:14', '2023-06-13T22:48', 'Seg 1/3'),
    ('Cyg X-1', 0.35, 2008501, '2023-06-13T23:05', '2023-06-14T16:10', ''),
    ('1E 2259+586', 3.4, 2007802, '2023-06-14T16:27', '2023-06-20T00:48', 'Seg 2/3'),
    ('Cyg X-1', 0.41, 2008601, '2023-06-20T01:05', '2023-06-20T21:21', ''),
    ('3C 454.3', 3.0, 2008901, '2023-06-20T21:40', '2023-06-26T21:25', 'Truncated by reset recovery'),
    ('1E 2259+586', 3.88, 2007803, '2023-07-01T16:03', '2023-07-07T17:54', 'Seg 3/3: Obs UID=2007899'),
    ('LMC X-3', 1.16, 2006501, '2023-07-07T18:41', '2023-07-09T13:43', 'Seg 1/2'),
    ('Her X-1', 1.74, 2003901, '2023-07-09T14:33', '2023-07-12T16:06', ''),
    ('LMC X-3', 5.4, 2006502, '2023-07-12T16:55', '2023-07-20T14:24', 'Seg 2/2: Obs UID=2006599'),
    ('Swift J0243.6+6124', 1.97, 2250701, '2023-07-20T15:09', '2023-07-23T15:56', 'Seg 1/3 ToO'),
    ('RCW 86', 1.16, 2001501, '2023-07-23T16:49', '2023-07-25T12:04', 'Seg 1/3'),
    ('Her X-1', 2.75, 2004101, '2023-07-25T12:44', '2023-07-30T12:22', ''),
    ('RCW 86', 1.92, 2001502, '2023-07-30T13:02', '2023-08-02T10:56', 'Seg 2/3'),
    ('Cir X-1', 1.56, 2002601, '2023-08-02T11:04', '2023-08-04T23:52', 'Seg 1/2'),
    ('RCW 86', 3.3, 2001503, '2023-08-04T23:59', '2023-08-09T22:34', 'Seg 3/3: Obs UID=2001599'),
    ('Swift J0243.6+6124', 0.92, 2250702, '2023-08-09T23:26', '2023-08-11T09:02', 'Seg 2/3 ToO'),
    ('Cir X-1', 1.64, 2002602, '2023-08-11T09:53', '2023-08-14T00:23', 'Seg 2/2: Obs UID=2002699'),
    ('1ES 1959+650', 3.67, 2250801, '2023-08-14T01:08', '2023-08-19T09:06', 'ToO (slow)'),
    ('4U 1624-49', 2.31, 2007301, '2023-08-19T09:50', '2023-08-23T08:56', ''),
    ('Swift J0243.6+6124', 1.57, 2250703, '2023-08-23T09:46', '2023-08-25T19:26', 'Seg 3/3: Obs UID=2250799 ToO'),
    ('RX J1713.7-3946', 1.57, 2001401, '2023-08-25T20:14', '2023-08-28T15:38', 'Seg 1/3'),
    ('Sco X-1', 0.32, 2002401, '2023-08-28T15:53', '2023-08-29T07:35', 'Atten'),
    ('RX J1713.7-3946', 2.65, 2001402, '2023-08-29T07:49', '2023-09-02T23:56', 'Seg 2/3'),
    ('Sgr A complex', 4.63, 2007901, '2023-09-03T00:06', '2023-09-07T19:24', 'Seg 1/3'),
    ('Swift J1727.8-1613', 0.23, 2250901, '2023-09-07T19:35', '2023-09-08T06:36', 'ToO Atten Obs 1/5'),
    ('Sgr A complex', 4.63, 2007902, '2023-09-08T06:46', '2023-09-16T16:40', 'Seg 2/3'),
    ('Swift J1727.8-1613', 0.46, 2251001, '2023-09-16T16:50', '2023-09-17T13:29', 'ToO Atten Obs 2/5'),
    ('RX J1713.7-3946', 5.85, 2001403, '2023-09-17T13:43', '2023-09-27T22:02', 'Seg 3/3: Obs UID=2001499'),
    ('Swift J1727.8-1613', 0.23, 2251101, '2023-09-27T22:15', '2023-09-28T09:15', 'ToO Atten Obs 3/5'),
    ('Sgr A complex', 3.24, 2007903, '2023-09-28T09:25', '2023-10-04T12:47', 'Seg 3/3 Obs UID=2007999'),
    ('Swift J1727.8-1613', 0.23, 2251201, '2023-10-04T12:57', '2023-10-04T23:58', 'ToO Atten Obs 4/5'),
    ('G21.5-0.9', 2.16, 2001101, '2023-10-05T00:10', '2023-10-09T01:50', 'Seg 1/3'),
    ('Crab', 0.69, 2006001, '2023-10-09T02:40', '2023-10-10T10:38', ''),
    ('Swift J1727.8-1613', 0.21, 2251201, '2023-10-10T11:30', '2023-10-10T22:39', 'ToO Atten Obs 5/5'),
    ('G21.5-0.9', 3.62, 2001102, '2023-10-10T22:50', '2023-10-17T16:37', 'Seg 2/3'),
    ('GX 13+1', 1.16, 2006801, '2023-10-17T16:45', '2023-10-19T21:30', ''),
    ('G21.5-0.9', 4.19, 2001103, '2023-10-19T21:38', '2023-10-27T15:28', 'Seg 3/3 Obs UID=2001199'),
    ('PKS 2155-304', 5.7, 2005601, '2023-10-27T15:47', '2023-11-07T00:09', ''),
    ('BL Lac', 6.18, 2009701, '2023-11-07T00:32', '2023-11-17T19:48', ''),
    ('Cyg X-3', 3.48, 2009101, '2023-11-17T19:59', '2023-11-23T23:08', ''),
    ('Vela Jr', 7.48, 2009601, '2023-11-24T00:00', '2023-12-06T21:15', 'Seg 1/2'),
    ('Mrk 421', 1.94, 2008101, '2023-12-06T21:41', '2023-12-10T08:42', 'Seg 1/4'),
    ('SMC X-1', 1.1, 2007001, '2023-12-10T09:28', '2023-12-11T22:00', ''),
    ('Mrk 421', 1.6, 2008102, '2023-12-11T22:47', '2023-12-14T20:03', 'Seg 2/4'),
    ('SMC X-1', 1.1, 2007101, '2023-12-14T20:50', '2023-12-16T06:45', ''),
    ('Mrk 421', 1.63, 2008103, '2023-12-16T07:32', '2023-12-19T05:48', 'Seg 3/4'),
    ('SMC X-1', 1.04, 2007201, '2023-12-19T06:35', '2023-12-20T17:14', ''),
    ('Mrk 421', 0.81, 2008104, '2023-12-20T18:01', '2023-12-22T07:31', 'Seg 4/4 Obs UID=2008199'),
    ('Vela Jr', 7.24, 2009602, '2023-12-22T07:57', '2024-01-03T20:35', 'Seg 2/2 Obs UID=2009699'),
    ('NGC 1068', 13.89, 2008001, '2024-01-03T21:14', '2024-01-29T19:16', ''),
    ('RCW 86', 3.17, 2009501, '2024-01-29T19:59', '2024-02-03T17:09', '')
)


# Full source list.
_ARTL_DATA = _Y1_ARTL_DATA + _Y2_ARTL_DATA
